#!/usr/bin/env python3
# coding: utf-8
# File: answer_search.py
# Author: lhy<lhy_in_blcu@126.com,https://huangyong.github.io>
# Date: 18-10-5

from sm4_utils import CryptSM4
from type_enum import word_type_enum, question_type_enum


class AnswerSearcher:
    def __init__(self, g):
        self.g = g
        self.num_limit = 20
        self.sm4 = CryptSM4('mkQXUpzFZpD3JLrw')

    '''执行cypher查询，并返回相应结果'''

    def search_main(self, sqls):
        final_answers = []
        for sql_ in sqls:
            question_type = sql_['question_type']
            queries = sql_['sql']
            answers = []
            for query in queries:
                ress = self.g.run(query).data()
                answers += ress
            final_answer = self.answer_prettify(question_type, answers)
            if final_answer:
                final_answers.append(final_answer)
        return final_answers

    '''根据对应的qustion_type，调用相应的回复模板'''

    def answer_prettify(self, question_type, answers):
        final_answer = []
        if not answers:
            return ''
        if question_type == question_type_enum.DISEASE_SYMPTOM:
            desc = [i['n.name'] for i in answers]
            subject = answers[0]['m.name']
            final_answer = '{0}的症状包括：{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.SYMPTOM_DISEASE:
            desc = [i['m.name'] for i in answers]
            subject = answers[0]['n.name']
            final_answer = '症状{0}可能染上的疾病有：{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.DISEASE_CAUSE:
            desc = [self.sm4.decryptSM4(i['m.cause']) for i in answers]
            subject = answers[0]['m.name']
            final_answer = '{0}可能的成因有：{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.DISEASE_PREVENT:
            desc = [self.sm4.decryptSM4(i['m.prevent']) for i in answers]
            subject = answers[0]['m.name']
            final_answer = '{0}的预防措施包括：{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.DISEASE_LASTTIME:
            desc = [self.sm4.decryptSM4(i['m.cure_lasttime']) for i in answers]
            subject = answers[0]['m.name']
            final_answer = '{0}治疗可能持续的周期为：{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.DISEASE_CUREWAY:
            desc = [self.sm4.decryptSM4(i['m.cure_way']) for i in answers]
            subject = answers[0]['m.name']
            final_answer = '{0}可以尝试如下治疗：{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.DISEASE_CUREPROB:
            desc = [self.sm4.decryptSM4(i['m.cured_prob']) for i in answers]
            subject = answers[0]['m.name']
            final_answer = '{0}治愈的概率为（仅供参考）：{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.DISEASE_EASYGET:
            desc = [self.sm4.decryptSM4(i['m.easy_get']) for i in answers]
            subject = answers[0]['m.name']

            final_answer = '{0}的易感人群包括：{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.DISEASE_DESC:
            desc = [i['m.desc'] for i in answers]
            subject = answers[0]['m.name']
            final_answer = '{0},熟悉一下：{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.DISEASE_ACOMPANY:
            desc1 = [i['n.name'] for i in answers]
            desc2 = [i['m.name'] for i in answers]
            subject = answers[0]['m.name']
            desc = [i for i in desc1 + desc2 if i != subject]
            final_answer = '{0}的症状包括：{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.DISEASE_NOT_FOOD:
            desc = [i['n.name'] for i in answers]
            subject = answers[0]['m.name']
            final_answer = '{0}忌食的食物包括有：{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.DISEASE_DO_FOOD:
            do_desc = [i['n.name'] for i in answers if i['r.name'] == '宜吃']
            recommand_desc = [i['n.name'] for i in answers if i['r.name'] == '推荐食谱']
            subject = answers[0]['m.name']
            final_answer = '{0}宜食的食物包括有：{1}\n推荐食谱包括有：{2}'.format(subject, ';'.join(list(set(do_desc))[:self.num_limit]), ';'.join(list(set(recommand_desc))[:self.num_limit]))

        elif question_type == question_type_enum.FOOD_NOT_DISEASE:
            desc = [i['m.name'] for i in answers]
            subject = answers[0]['n.name']
            final_answer = '患有{0}的人最好不要吃{1}'.format('；'.join(list(set(desc))[:self.num_limit]), subject)

        elif question_type == question_type_enum.FOOD_DO_DISEASE:
            desc = [i['m.name'] for i in answers]
            subject = answers[0]['n.name']
            final_answer = '患有{0}的人建议多试试{1}'.format('；'.join(list(set(desc))[:self.num_limit]), subject)

        elif question_type == 'disease_drug':
            desc = [i['n.name'] for i in answers]
            subject = answers[0]['m.name']
            final_answer = '{0}通常的使用的药品包括：{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.DRUG_DISEASE:
            desc = [i['m.name'] for i in answers]
            subject = answers[0]['n.name']
            final_answer = '{0}主治的疾病有{1},可以试试'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.DISEASE_CHECK:
            desc = [i['n.name'] for i in answers]
            subject = answers[0]['m.name']
            final_answer = '{0}通常可以通过以下方式检查出来：{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.CHECK_DISEASE:
            desc = [i['m.name'] for i in answers]
            subject = answers[0]['n.name']
            final_answer = '通常可以通过{0}检查出来的疾病有{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.FOODMATERIAL_NOT_CROWD:
            desc = [i['n.name'] for i in answers]
            subject = answers[0]['m.name']
            final_answer = '患有{1}等疾病的人群不适宜食用{0}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.FOODMATERIAL_DO_CROWD:
            desc = [i['n.name'] for i in answers]
            subject = answers[0]['m.name']
            final_answer = '患有{1}等疾病的人群适宜食用{0}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.FOODPROCESS_NOT_CROWD:
            desc = [i['n.name'] for i in answers]
            subject = answers[0]['m.name']
            final_answer = '患有{1}等疾病的人群不适宜食用{0}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.FOODPROCESS_DO_CROWD:
            desc = [i['n.name'] for i in answers]
            subject = answers[0]['m.name']
            final_answer = '患有{1}等疾病的人群适宜食用{0}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.FOODPROCESS_LEVEL_HIGH or question_type == question_type_enum.FOODMATERIAL_LEVEL_HIGH:
            desc = [i['n.name'] for i in answers]
            subject = answers[0]['m.name']
            final_answer = '{0}中的{1}较高'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.FOODPROCESS_LEVEL_MID or question_type == question_type_enum.FOODMATERIAL_LEVEL_MID:
            desc = [i['n.name'] for i in answers]
            subject = answers[0]['m.name']
            final_answer = '{0}中的{1}适中'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.FOODPROCESS_LEVEL_LOW or question_type == question_type_enum.FOODMATERIAL_LEVEL_LOW:
            desc = [i['n.name'] for i in answers]
            subject = answers[0]['m.name']
            final_answer = '{0}中的{1}较低'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.INDEX_LEVEL_HIGH:
            desc = [i['m.name'] for i in answers]
            subject = answers[0]['n.name']
            final_answer = '{0}较高的食物有：{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.INDEX_LEVEL_MID:
            desc = [i['m.name'] for i in answers]
            subject = answers[0]['n.name']
            final_answer = '{0}适中的食物有：{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.INDEX_LEVEL_LOW:
            desc = [i['m.name'] for i in answers]
            subject = answers[0]['n.name']
            final_answer = '{0}较低的食物有：{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.FOODPROCESS_LEVEL_CONTENT:
            final_answer = ''
            for i in answers:
                level = '未知'
                if i['type'] == 'HIGH_TO':
                    level = '较高'
                elif i['type'] == 'MIDDLE_TO':
                    level = '适中'
                elif i['type'] == 'LOW_TO':
                    level = '较低'
                final_answer += f"{i['m.name']}的{i['n.name']}属于{level}的水平；"

        elif question_type == question_type_enum.FOODPROCESS_COOKING_SPECIALTY or question_type == question_type_enum.FOODMATERIAL_COOKING_SPECIALTY:
            desc = [i['n.name'] for i in answers]
            subject = answers[0]['m.name']
            final_answer = '{0}的烹饪特点是:{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.FOODPROCESS_FOOD_SPECIALTY or question_type == question_type_enum.FOODMATERIAL_FOOD_SPECIALTY:
            desc = [i['n.name'] for i in answers]
            subject = answers[0]['m.name']
            final_answer = '{0}的膳食特点是:{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.FOODPROCESS_PROVINCE or question_type == question_type_enum.FOODMATERIAL_PROVINCE:
            desc = [i['n.name'] for i in answers]
            subject = answers[0]['m.name']
            final_answer = '{0}所属地域包括:{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.PROVINCE_FOOD:
            desc = [i['m.name'] for i in answers]
            subject = answers[0]['n.name']
            final_answer = '{0}的菜品有:{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.FOODPROCESS_UNIT or question_type == question_type_enum.FOODMATERIAL_UNIT:
            desc = [i['n.name'] for i in answers]
            subject = answers[0]['m.name']
            final_answer = '{0}的扩展单位是:{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.FOODPROCESS_HIGH_CONT or question_type == question_type_enum.FOODMATERIAL_HIGH_CONT:
            desc = [i['n.name'] for i in answers]
            subject = answers[0]['m.name']
            final_answer = '{0}富含的元素有:{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.NUTRIENT_HIGH_CONT:
            desc = [i['m.name'] for i in answers]
            subject = answers[0]['n.name']
            final_answer = '富含{0}的食物有:{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.FOODPROCESS_LOW_CONT or question_type == question_type_enum.FOODMATERIAL_LOW_CONT:
            desc = [i['n.name'] for i in answers]
            subject = answers[0]['m.name']
            final_answer = '{0}含量低的元素有:{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.NUTRIENT_LOW_CONT:
            desc = [i['m.name'] for i in answers]
            subject = answers[0]['n.name']
            final_answer = '含低量{0}的食物有:{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))

        elif question_type == question_type_enum.FOODPROCESS_ALIAS:
            desc = [i['n.name'] for i in answers]
            subject = answers[0]['m.name']
            final_answer = '{0}的别名还有:{1}'.format(subject, '；'.join(list(set(desc))[:self.num_limit]))


        return final_answer


if __name__ == '__main__':
    searcher = AnswerSearcher()
