#!/usr/bin/env python3
# coding: utf-8
# File: chatbot_graph.py
# Author: lhy<lhy_in_blcu@126.com,https://huangyong.github.io>
# Date: 18-10-4

import gradio as gr
from py2neo import Graph
from question_classifier import *
from question_parser import *
from answer_search import *

'''问答类'''


class ChatBotGraph:
    def __init__(self):
        self.g = Graph(
            'http://127.0.0.1:7474',
            user="neo4j",
            password="neo4j")
        self.classifier = QuestionClassifier(self.g)
        self.parser = QuestionPaser()
        self.searcher = AnswerSearcher(self.g)

    def chat_main(self, sent):
        answer = '您好，我是医药智能助理-小益，希望可以帮到您。如果没答上来，可联系人工客服。祝您身体棒棒！'
        res_classify = self.classifier.classify(sent)
        if not res_classify:
            return answer
        res_sql = self.parser.parser_main(res_classify)
        final_answers = self.searcher.search_main(res_sql)
        if not final_answers:
            return answer
        else:
            return '\n'.join(final_answers)


# if __name__ == '__main__':
#     handler = ChatBotGraph()
#     while 1:
#         question = input('用户:')
#         answer = handler.chat_main(question)
#         print('小益:', answer)

if __name__ == '__main__':
    handler = ChatBotGraph()
    # 接口创建函数
    # fn设置处理函数，inputs设置输入接口组件，outputs设置输出接口组件
    # fn,inputs,outputs都是必填函数
    demo = gr.Interface(fn=handler.chat_main, inputs="text", outputs="text")

    demo.launch(share=True)
