#!/usr/bin/env python3
# coding: utf-8
# File: question_parser.py
# Author: lhy<lhy_in_blcu@126.com,https://huangyong.github.io>
# Date: 18-10-4


from type_enum import word_type_enum, question_type_enum


class QuestionPaser:
    '''构建实体节点'''

    def build_entitydict(self, args):
        entity_dict = {}
        for arg, types in args.items():
            for type in types:
                if type not in entity_dict:
                    entity_dict[type] = [arg]
                else:
                    entity_dict[type].append(arg)

        return entity_dict

    '''解析主函数'''

    def parser_main(self, res_classify):
        args = res_classify['args']
        entity_dict = self.build_entitydict(args)
        question_types = res_classify['question_types']
        sqls = []
        for question_type in question_types:
            sql_ = {}
            sql_['question_type'] = question_type
            sql = []

            # 查询疾病有哪些症状
            if question_type == question_type_enum.DISEASE_SYMPTOM:
                sql = ["MATCH (m:Disease)-[r:has_symptom]->(n:Symptom) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.DISEASE)]

            # 查询症状会导致哪些疾病
            elif question_type == question_type_enum.SYMPTOM_DISEASE:
                sql = ["MATCH (m:Disease)-[r:has_symptom]->(n:Symptom) where n.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.SYMPTOM)]

            # 查询疾病的原因
            elif question_type == question_type_enum.DISEASE_CAUSE:
                sql = ["MATCH (m:Disease) where m.name = '{0}' return m.name, m.cause".format(i) for i in entity_dict.get(word_type_enum.DISEASE)]

            # 查询疾病的并发症
            elif question_type == question_type_enum.DISEASE_ACOMPANY:
                sql1 = ["MATCH (m:Disease)-[r:acompany_with]->(n:Disease) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.DISEASE)]
                sql2 = ["MATCH (m:Disease)-[r:acompany_with]->(n:Disease) where n.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.DISEASE)]
                sql = sql1 + sql2

            # 查询疾病的忌口
            elif question_type == question_type_enum.DISEASE_NOT_FOOD:
                sql = ["MATCH (m:Disease)-[r:no_eat]->(n:Food) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.DISEASE)]

            # 查询疾病建议吃的东西
            elif question_type == question_type_enum.DISEASE_DO_FOOD:
                sql1 = ["MATCH (m:Disease)-[r:do_eat]->(n:Food) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.DISEASE)]
                sql2 = ["MATCH (m:Disease)-[r:recommend_eat]->(n:Food) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.DISEASE)]
                sql = sql1 + sql2

            # 已知忌口查疾病
            elif question_type == question_type_enum.FOOD_NOT_DISEASE:
                sql = ["MATCH (m:Disease)-[r:no_eat]->(n:Food) where n.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD)]

            # 已知推荐查疾病
            elif question_type == question_type_enum.FOOD_DO_DISEASE:
                sql1 = ["MATCH (m:Disease)-[r:do_eat]->(n:Food) where n.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD)]
                sql2 = ["MATCH (m:Disease)-[r:recommend_eat]->(n:Food) where n.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD)]
                sql = sql1 + sql2

            # 查询疾病常用药品－药品别名记得扩充
            elif question_type == question_type_enum.DISEASE_DRUG:
                sql1 = ["MATCH (m:Disease)-[r:common_drug]->(n:Drug) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.DISEASE)]
                sql2 = ["MATCH (m:Disease)-[r:recommand_drug]->(n:Drug) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.DISEASE)]
                sql = sql1 + sql2

            # 已知药品查询能够治疗的疾病
            elif question_type == question_type_enum.DRUG_DISEASE:
                sql1 = ["MATCH (m:Disease)-[r:common_drug]->(n:Drug) where n.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.DRUG)]
                sql2 = ["MATCH (m:Disease)-[r:recommand_drug]->(n:Drug) where n.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.DRUG)]
                sql = sql1 + sql2

            # 查询疾病应该进行的检查
            elif question_type == question_type_enum.DISEASE_CHECK:
                sql = ["MATCH (m:Disease)-[r:need_check]->(n:Check) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.DISEASE)]

            # 已知检查查询疾病
            elif question_type == question_type_enum.CHECK_DISEASE:
                sql = ["MATCH (m:Disease)-[r:need_check]->(n:Check) where n.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.CHECK)]

            # 查询疾病的防御措施
            elif question_type == question_type_enum.DISEASE_PREVENT:
                sql = ["MATCH (m:Disease) where m.name = '{0}' return m.name, m.prevent".format(i) for i in entity_dict.get(word_type_enum.DISEASE)]

            # 查询疾病的持续时间
            elif question_type == question_type_enum.DISEASE_LASTTIME:
                sql = ["MATCH (m:Disease) where m.name = '{0}' return m.name, m.cure_lasttime".format(i) for i in entity_dict.get(word_type_enum.DISEASE)]

            # 查询疾病的治疗方式
            elif question_type == question_type_enum.DISEASE_CUREWAY:
                sql = ["MATCH (m:Disease) where m.name = '{0}' return m.name, m.cure_way".format(i) for i in entity_dict.get(word_type_enum.DISEASE)]

            # 查询疾病的治愈概率
            elif question_type == question_type_enum.DISEASE_CUREPROB:
                sql = ["MATCH (m:Disease) where m.name = '{0}' return m.name, m.cured_prob".format(i) for i in entity_dict.get(word_type_enum.DISEASE)]

            # 查询疾病的易发人群
            elif question_type == question_type_enum.DISEASE_EASYGET:
                sql = ["MATCH (m:Disease) where m.name = '{0}' return m.name, m.easy_get".format(i) for i in entity_dict.get(word_type_enum.DISEASE)]

            # 查询疾病的相关介绍
            elif question_type == question_type_enum.DISEASE_DESC:
                sql = ["MATCH (m:Disease) where m.name = '{0}' return m.name, m.desc".format(i) for i in entity_dict.get(word_type_enum.DISEASE)]

            # 已知忌口查人群
            elif question_type == question_type_enum.FOODMATERIAL_NOT_CROWD:
                sql = ["MATCH (m:FoodMaterial)-[r:taboo_crowd]->(n:Crowd) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_MATERIAL)]

            # 已知推荐食品查人群
            elif question_type == question_type_enum.FOODMATERIAL_DO_CROWD:
                sql = ["MATCH (m:FoodMaterial)-[r:recommend_crowd]->(n:Crowd) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_MATERIAL)]

            # 已知忌口查人群
            elif question_type == question_type_enum.FOODPROCESS_NOT_CROWD:
                sql = ["MATCH (m:FoodProcess)-[r:taboo_crowd]->(n:Crowd) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_PROCESS)]

            # 已知推荐食品查人群
            elif question_type == question_type_enum.FOODPROCESS_DO_CROWD:
                sql = ["MATCH (m:FoodProcess)-[r:recommend_crowd]->(n:Crowd) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_PROCESS)]

            # 食物、含量关系、Level，已知1，3查2
            elif question_type == question_type_enum.FOODPROCESS_LEVEL_CONTENT:
                sql = ["MATCH (m:FoodProcess)-[r]->(n:IndexLevel) where m.name = '{0}' AND n.name contains '{1}' return m.name, type(r) AS type, n.name".format(i, j) for i in
                       entity_dict.get(word_type_enum.FOOD_PROCESS) for j in entity_dict.get(word_type_enum.INDEX_LEVEL)]

            elif question_type == question_type_enum.FOODMATERIAL_LEVEL_CONTENT:
                sql = ["MATCH (m:FoodMaterial)-[r]->(n:IndexLevel) where m.name = '{0}' AND n.name contains '{1}' return m.name, type(r) AS type, n.name".format(i, j) for i in
                       entity_dict.get(word_type_enum.FOOD_MATERIAL) for j in entity_dict.get(word_type_enum.INDEX_LEVEL)]

            # 食物、含量关系、Level，已知1，2查3
            elif question_type == question_type_enum.FOODPROCESS_LEVEL_HIGH:
                sql = ["MATCH (m:FoodProcess)-[r:HIGH_TO]->(n:IndexLevel) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_PROCESS)]

            elif question_type == question_type_enum.FOODPROCESS_LEVEL_MID:
                sql = ["MATCH (m:FoodProcess)-[r:MIDDLE_TO]->(n:IndexLevel) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_PROCESS)]

            elif question_type == question_type_enum.FOODPROCESS_LEVEL_LOW:
                sql = ["MATCH (m:FoodProcess)-[r:LOW_TO]->(n:IndexLevel) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_PROCESS)]

            elif question_type == question_type_enum.FOODMATERIAL_LEVEL_HIGH:
                sql = ["MATCH (m:FoodMaterial)-[r:HIGH_TO]->(n:IndexLevel) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_MATERIAL)]

            elif question_type == question_type_enum.FOODMATERIAL_LEVEL_MID:
                sql = ["MATCH (m:FoodMaterial)-[r:MIDDLE_TO]->(n:IndexLevel) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_MATERIAL)]

            elif question_type == question_type_enum.FOODMATERIAL_LEVEL_LOW:
                sql = ["MATCH (m:FoodMaterial)-[r:LOW_TO]->(n:IndexLevel) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_MATERIAL)]

            # 食物、含量关系、Level，已知3，2查1
            elif question_type == question_type_enum.INDEX_LEVEL_HIGH:
                sql = ["MATCH (m:FoodProcess)-[r:HIGH_TO]->(n:IndexLevel) where n.name contains '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.INDEX_LEVEL)]
                sql += ["MATCH (m:FoodMaterial)-[r:HIGH_TO]->(n:IndexLevel) where n.name contains '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.INDEX_LEVEL)]

            elif question_type == question_type_enum.INDEX_LEVEL_MID:
                sql = ["MATCH (m:FoodProcess)-[r:MIDDLE_TO]->(n:IndexLevel) where n.name contains '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.INDEX_LEVEL)]
                sql += ["MATCH (m:FoodMaterial)-[r:MIDDLE_TO]->(n:IndexLevel) where n.name contains '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.INDEX_LEVEL)]

            elif question_type == question_type_enum.INDEX_LEVEL_LOW:
                sql = ["MATCH (m:FoodProcess)-[r:LOW_TO]->(n:IndexLevel) where n.name contains '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.INDEX_LEVEL)]
                sql += ["MATCH (m:FoodMaterial)-[r:LOW_TO]->(n:IndexLevel) where n.name contains '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.INDEX_LEVEL)]

            # 已知食物找烹饪特点
            elif question_type == question_type_enum.FOODPROCESS_COOKING_SPECIALTY:
                sql = ["MATCH (m:FoodProcess)-[r:cooking_specialty]->(n:CookingSpecialty) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_PROCESS)]

            elif question_type == question_type_enum.FOODMATERIAL_COOKING_SPECIALTY:
                sql = ["MATCH (m:FoodMaterial)-[r:cooking_specialty]->(n:CookingSpecialty) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_MATERIAL)]

            # 已知食物找膳食特点
            elif question_type == question_type_enum.FOODPROCESS_FOOD_SPECIALTY:
                sql = ["MATCH (m:FoodProcess)-[r:food_specialty]->(n:FoodSpecialty) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_PROCESS)]

            elif question_type == question_type_enum.FOODMATERIAL_FOOD_SPECIALTY:
                sql = ["MATCH (m:FoodMaterial)-[r:food_specialty]->(n:FoodSpecialty) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_MATERIAL)]

            # 已知食物找地域
            elif question_type == question_type_enum.FOODPROCESS_PROVINCE:
                sql = ["MATCH (m:FoodProcess)-[r:province_of]->(n:Province) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_PROCESS)]

            elif question_type == question_type_enum.FOODMATERIAL_PROVINCE:
                sql = ["MATCH (m:FoodMaterial)-[r:province_of]->(n:Province) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_MATERIAL)]

            # 已知地域找食物
            elif question_type == question_type_enum.PROVINCE_FOOD:
                sql = ["MATCH (m:FoodProcess)-[r:province_of]->(n:Province) where n.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.PROVINCE)]
                sql += ["MATCH (m:FoodMaterial)-[r:province_of]->(n:Province) where n.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.PROVINCE)]

            # 已知食物查单位
            elif question_type == question_type_enum.FOODPROCESS_UNIT:
                sql = ["MATCH (m:FoodProcess)-[r:extension_unit]->(n:ExtensionUnits) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_PROCESS)]

            elif question_type == question_type_enum.FOODMATERIAL_UNIT:
                sql = ["MATCH (m:FoodMaterial)-[r:extension_unit]->(n:ExtensionUnits) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_MATERIAL)]

            # 已知食物查富含的元素
            elif question_type == question_type_enum.FOODPROCESS_HIGH_CONT:
                sql = ["MATCH (m:FoodProcess)-[r:high_content]->(n:Nutrient) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_PROCESS)]

            elif question_type == question_type_enum.FOODMATERIAL_HIGH_CONT:
                sql = ["MATCH (m:FoodMaterial)-[r:high_content]->(n:Nutrient) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_MATERIAL)]

            # 已知富含的元素查食物
            elif question_type == question_type_enum.NUTRIENT_HIGH_CONT:
                sql = ["MATCH (m:FoodProcess)-[r:high_content]->(n:Nutrient) where n.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.NUTRIENT)]
                sql += ["MATCH (m:FoodMaterial)-[r:high_content]->(n:Nutrient) where n.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.NUTRIENT)]

            # 已知食物查低量的元素
            elif question_type == question_type_enum.FOODPROCESS_LOW_CONT:
                sql = ["MATCH (m:FoodProcess)-[r:low_content]->(n:Nutrient) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_PROCESS)]

            elif question_type == question_type_enum.FOODMATERIAL_LOW_CONT:
                sql = ["MATCH (m:FoodMaterial)-[r:low_content]->(n:Nutrient) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_MATERIAL)]

            # 已知低量的元素查食物
            elif question_type == question_type_enum.NUTRIENT_LOW_CONT:
                sql = ["MATCH (m:FoodProcess)-[r:low_content]->(n:Nutrient) where n.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.NUTRIENT)]
                sql += ["MATCH (m:FoodMaterial)-[r:low_content]->(n:Nutrient) where n.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.NUTRIENT)]

            # 已知菜品查别名
            elif question_type == question_type_enum.FOODPROCESS_ALIAS:
                sql = ["MATCH (m:FoodProcess)-[r:alias_of]->(n:Alias) where m.name = '{0}' return m.name, r.name, n.name".format(i) for i in entity_dict.get(word_type_enum.FOOD_PROCESS)]

            if sql:
                sql_['sql'] = sql

                sqls.append(sql_)

        return sqls


if __name__ == '__main__':
    handler = QuestionPaser()
